package cc.fascinated.bat.features.namehistory.profile.guild;

import cc.fascinated.bat.common.Profile;
import cc.fascinated.bat.features.namehistory.NameHistoryFeature;
import cc.fascinated.bat.features.namehistory.TrackedName;
import cc.fascinated.bat.model.BatUser;

import java.util.*;

/**
 * @author Fascinated (fascinated7)
 */
public class NameHistoryProfile extends Profile {
    private Map<String, List<TrackedName>> nameHistory;

    public NameHistoryProfile() {
        super("name-history");
    }

    /**
     * Gets the name history of the user
     *
     * @param user the user to get the name history of
     * @return the name history of the user
     */
    public List<TrackedName> getNameHistory(BatUser user) {
        if (this.nameHistory == null) {
            this.nameHistory = new HashMap<>();
        }
        if (!this.nameHistory.containsKey(user.getId())) {
            this.nameHistory.put(user.getId(), new LinkedList<>());
        }
        return this.nameHistory.get(user.getId());
    }

    /**
     * Gets the name history of the user sorted
     *
     * @param user the user to get the name history of
     * @return the name history of the user sorted
     */
    public List<TrackedName> getNameHistorySorted(BatUser user) {
        return getNameHistory(user).stream().sorted((o1, o2) -> o2.getChangedDate().compareTo(o1.getChangedDate())).toList();
    }

    /**
     * Adds a name to the name history
     *
     * @param user the user to add the name to
     * @param name the name to add
     */
    public void addName(BatUser user, String name) {
        getNameHistory(user).add(new TrackedName(name, new Date()));
        cleanup();
    }

    /**
     * Cleans up the name history
     * <p>
     * This will remove any names that are not within
     * the size limit of the name history
     * </p>
     */
    private void cleanup() {
        for (String userId : this.nameHistory.keySet()) {
            List<TrackedName> trackedNames = this.nameHistory.get(userId);
            if (trackedNames.size() > NameHistoryFeature.NAME_HISTORY_SIZE) {
                trackedNames.sort((o1, o2) -> o2.getChangedDate().compareTo(o1.getChangedDate()));
                trackedNames.subList(NameHistoryFeature.NAME_HISTORY_SIZE, trackedNames.size()).clear();
            }
        }
    }

    @Override
    public void reset() {
        this.nameHistory = null;
    }
}
